@@ -638,7 +638,7 @@ class ElementBuilder:
             self.expression = "value"
         else:
             self.pre_expression = ""
-            # NUMPY: reshape to the final shape if needed
+            # NUMPY: reshape to the final shape if needed)
             # expressions[0]["expr"].reshape(self.section.subscripts, {})
             if not expressions[0]["expr"].subscripts and self.subscripts:
                 # Updimension the return value to an array
@@ -660,6 +660,24 @@ class ElementBuilder:
         self.subtype = ", ".join(
             set(component.subtype for component in self.components)
         )
+        if ", " in self.type:
+            warn(
+                f"Variable '{self.name}' is defined with different types:"
+                f" '{self.type}'. This may cause bugs when trying to"
+                "change its value or applying other methods from the "
+                "pysd.py_backend.model.Model class. Running the model "
+                "without modifying this variable should not cause any "
+                "bug."
+            )
+        elif ", " in self.subtype:
+            warn(
+                f"Variable '{self.name}' is defined with different subtypes:"
+                f" '{self.subtype}'. This may cause bugs when trying to"
+                "change its value or applying other methods from the "
+                "pysd.py_backend.model.Model class. Running the model "
+                "without modifying this variable should not cause any "
+                "bug."
+            )
 
     def _manage_multi_def(self, expression: dict) -> str:
         """
@@ -722,13 +740,7 @@ class ElementBuilder:
 
         # Get the objects to create as string
         objects = "\n\n".join([
-            value["expression"] % {
-                "final_subs":
-                self.section.subscripts.simplify_subscript_input(
-                    value.get("final_subs", {}))[1]
-            }   # Replace the final subs in the objects that merge
-                # several components
-            for value in self.objects.values()
+            value["expression"] for value in self.objects.values()
             if value["expression"] is not None
         ])
 
