@@ -5,6 +5,7 @@ Several methods and propierties are inherited from Macro class, which
 allows integrating a model or a Macro expression (set of functions in
 a separate file).
 """
+import time
 import warnings
 import inspect
 import pickle
@@ -19,7 +20,8 @@ from pysd._version import __version__
 
 from . import utils
 from .statefuls import DynamicStateful, Stateful
-from .external import External, Excels
+from .external import External, Excels, ExtLookup, ExtData
+
 from .cache import Cache, constant_cache
 from .data import TabData
 from .lookups import HardcodedLookups
@@ -57,8 +59,8 @@ class Macro(DynamicStateful):
         Time to set at the begginning of the Macro. Default is None.
     data_files: dict or list or str or None
         The dictionary with keys the name of file and variables to
-        load the data from there. Or the list of names or name of the
-        file to search the data in. Only works for TabData type object
+        load the data from. Or the list of names or name of the file
+        to search the data in. Only works for TabData type object
         and it is neccessary to provide it. Default is None.
     py_name: str or None
         The name of the Macro object. Default is None.
@@ -453,13 +455,7 @@ class Macro(DynamicStateful):
 
         if not self.external_loaded:
             # Initialize external elements
-            for element in self._external_elements:
-                element.initialize()
-
-            # Remove Excel data from memory
-            Excels.clean()
-
-            self.external_loaded = True
+            self.initialize_external_data()
 
         # Initialize stateful objects
         for element_name in self.initialize_order:
@@ -532,6 +528,291 @@ class Macro(DynamicStateful):
         self.set_stateful(stateful_dict)
         self.time.set_control_vars(initial_time=time)
 
+    def initialize_external_data(self, externals=None):
+
+        """
+        Initializes external data.
+
+        If a path to a netCDF file containing serialized values of some or
+        all of the model external data is passed in the external argument,
+        those will be loaded from the file.
+
+        To get the full performance gain of loading the externals from a netCDF
+        file, the model should be loaded with initialize=False first.
+
+        Parameters
+        ----------
+        externals: str or pathlib.Path (optional)
+            Path to the netCDF file that contains the model external objects.
+
+        Returns
+        -------
+        None
+
+        """
+
+        if not externals:
+            for ext in self._external_elements:
+                ext.initialize()
+
+            # Remove Excel data from memory
+            Excels.clean()
+
+            self.external_loaded = True
+
+            return
+
+        externals = Path(externals)
+
+        if not externals.is_file():
+            raise FileNotFoundError(f"Invalid file path ({str(externals)})")
+
+        ds = xr.open_dataset(externals)
+
+        for ext in self._external_elements:
+            if ext.py_name in ds.data_vars.keys():
+                # Initialize external from nc file
+                da = ds.data_vars[ext.py_name]
+                if isinstance(ext, ExtData):
+                    # Rename again time dimension
+                    time_dim = [dim for dim in da.dims if dim.startswith(
+                        "time_#")][0]
+                    da = da.rename({time_dim: "time"})
+                elif isinstance(ext, ExtLookup):
+                    # Rename again lookup_dim dimension
+                    lookup_dim = [dim for dim in da.dims if dim.startswith(
+                        "lookup_dim_#")][0]
+                    da = da.rename({lookup_dim: "lookup_dim"})
+                # Assign the value
+                if da.dims:
+                    ext.data = da
+                else:
+                    ext.data = float(da.data)
+            else:
+                # Initialize external from original file
+                ext.initialize()
+
+        Excels.clean()
+
+        self.external_loaded = True
+
+    def serialize_externals(self, export_path="externals.nc",
+                            include_externals="all", exclude_externals=None):
+        """
+        Stores a netCDF file with the data and metadata for all model external
+        objects.
+
+        This method is useful for models with lots of external inputs, which
+        are slow to load into memory. Once exported, the resulting netCDF file
+        can be passed as argument to the initialize_external_data method.
+
+        Names of variables should be those in the model (python safe,
+        without the _ext_type_ string in front).
+
+        Parameters
+        ----------
+        export_path: str or pathlib.Path (optional)
+            Path of the resulting *.nc* file.
+        include_externals: list or str (optional)
+            External objects to export to netCDF.
+            If 'all', then all externals are exported to the *.nc* file.
+            The argument also accepts a list containing spreadsheet file names,
+            external variable names or a combination of both. If a spreadsheet
+            file path is passed, all external objects defined in it will be
+            included in the *.nc* file. Spreadsheet tab names are not currently
+            supported, because the same may be used in different files.
+            Better customisation can be achieved by combining the
+            include_externals and exclude_externals (see description below)
+            arguments.
+        exclude_externals: list or None (optional)
+            Exclude external objects from being included in the exported nc
+            file. It accepts either variable names, spreadsheet files or a
+            combination of both.
+
+        Returns
+        -------
+        None
+
+        """
+        data = {}
+        metadata = {}
+
+        lookup_dims = utils.UniqueDims("lookup_dim")
+        data_dims = utils.UniqueDims("time")
+
+        if isinstance(export_path, str):
+            export_path = Path(export_path)
+
+        if not include_externals:
+            raise ValueError("include_externals argument must not be None.")
+
+        # Generate a Dataframe to make simpler the search of external
+        # objects to include
+        # TODO include also checking the original name
+        py_names = []
+        externals_dict = {"py_var_name": [], "file": [], "ext": []}
+        for ext in self._external_elements:
+            py_names.append(ext.py_name)
+            externals_dict["py_var_name"].append(
+                self.__get_varname_from_ext_name(ext.py_name))
+            externals_dict["file"].append(set(ext.files))
+            externals_dict["ext"].append(ext)
+        exts_df = pd.DataFrame(index=py_names, data=externals_dict)
+
+        if include_externals != "all":
+            if not isinstance(include_externals, (list, set)):
+                raise TypeError(
+                    "include_externals must be 'all', or a list, or a set.")
+
+            # subset only to the externals to include
+            exts_df = exts_df[[
+                name in include_externals
+                or var_name in include_externals
+                or bool(file.intersection(include_externals))
+                for name, (var_name, file)
+                in exts_df[["py_var_name", "file"]].iterrows()
+            ]]
+
+        if exclude_externals:
+            if not isinstance(exclude_externals, (list, set)):
+                raise TypeError("exclude_externals must be a list or a set.")
+
+            # subset only to the externals to include
+            exts_df = exts_df[[
+                name not in exclude_externals
+                and var_name not in exclude_externals
+                and not bool(file.intersection(exclude_externals))
+                for name, (var_name, file)
+                in exts_df[["py_var_name", "file"]].iterrows()
+            ]]
+
+        for _, (ext, var_name) in exts_df[["ext", "py_var_name"]].iterrows():
+            self.__include_for_serialization(
+                ext, var_name, data, metadata, lookup_dims, data_dims
+            )
+
+        # create description to be used as global attribute of the dataset
+        description = {
+            "description": f"External objects for {self.py_model_file} "
+                           f"exported on {time.ctime(time.time())} "
+                           f"using PySD version {__version__}"
+        }
+
+        # create Dataset
+        ds = xr.Dataset(data_vars=data, attrs=description)
+
+        # add data_vars attributes
+        for key, values in metadata.items():
+            ds[key].attrs = values
+
+        ds.to_netcdf(export_path)
+
+    def __include_for_serialization(self, ext, py_name_clean, data, metadata,
+                                    lookup_dims, data_dims):
+        """
+        Initialize the external object and get the data and metadata for
+        inclusion in the netCDF.
+
+        This function updates the metadata dict with the metadata corresponding
+        to each external object, which is collected from model.doc. It also
+        updates the data dict, with the data from the external object (ext).
+
+        It renames the "time" dimension of ExtData types by appending _#
+        followed by a unique number at the end of it. For large models, this
+        prevents having an unnecessary large time dimension, which then causes
+        all ExtData objects to have many nans when stored in a xarray Dataset.
+        It does the same for the "lookup_dim" of all ExtLookup objects.
+
+        NOTE: Though subscripts can be read from Excel, they are hardcoded
+        during the model building process. Therefore they will not be
+        serialized.
+
+        Parameters
+        ----------
+        ext: pysd.py_backend.externals.External
+            External object. It can be any of the External subclasses
+            (ExtConstant, ExtData, ExtLookup)
+        py_name_clean: str
+            Name of the variable without _ext_[constant|data|lookup] prefix.
+        data: dict
+            Collects all the data for each external, which is later used to
+            build the xarray Dataset.
+        metadata: dict
+            Collects the metadata for each external, which is later included
+            as data_vars attributes in the xarray Dataset.
+        lookup_dims: utils.UniqueDims
+            UniqueDims object for "lookup_dim" dimension.
+        data_dims: utils.UniqueDims
+            UniqueDims object for "time" dimension.
+
+        Returns
+        -------
+        None
+
+        """
+        ext.initialize()
+
+        # collecting variable metadata from model._doc
+        var_meta = {
+            col:
+            self._doc.loc[self._doc["Py Name"] == py_name_clean, col].values[0]
+            or "Missing"
+            for col in self._doc.columns
+        }
+        var_meta["files"] = ";".join(ext.files)
+        var_meta["sheets"] = ";".join(ext.sheets)
+        var_meta["cells"] = ";".join(ext.cells)
+        # TODO: add also time_row_or_cols
+
+        da = ext.data
+
+        # Renaming shared dims by all ExtData ("time") and ExtLookup
+        # ("lookup_dim") external objects
+        if isinstance(ext, ExtData):
+            new_name = data_dims.name_new_dim("time", da.coords["time"].values)
+            da = da.rename({"time": new_name})
+        if isinstance(ext, ExtLookup):
+            new_name = lookup_dims.name_new_dim("lookup_dim",
+                                                da.coords["lookup_dim"].values)
+            da = da.rename({"lookup_dim": new_name})
+
+        metadata.update({ext.py_name: var_meta})
+        data.update({ext.py_name: da})
+
+        # TODO use a logger
+        print(f"Finished processing variable {py_name_clean}.")
+
+    def __get_varname_from_ext_name(self, varname):
+        """
+        Returns the name of the variable that depends on the external object
+        named varname. If that is not possible (see warning in the code to
+        understand when that may happen), it gets the name by removing the
+        _ext_[constant|data|lookup] prefix from the varname.
+
+        Parameters
+        ----------
+        varname: str
+            Variable name to which the External object is assigned.
+
+        Returns
+        -------
+        var: str
+            Name of the variable that calls the variable with name varname.
+
+        """
+        for var, deps in self._dependencies.items():
+            for _, ext_name in deps.items():
+                if varname == ext_name:
+                    return var
+
+        warnings.warn(
+            f"No variable depends upon '{varname}'. This is likely due "
+            f"to the fact that '{varname}' is defined using a mix of "
+            "DATA and CONSTANT. Though Vensim allows it, it is "
+            "not recommended."
+        )
+        return "_".join(varname.split("_")[3:])
+
     def get_args(self, param):
         """
         Returns the arguments of a model element.
@@ -1287,18 +1568,18 @@ class Model(Macro):
         UserWarning: Selecting submodel, to run the full model again use model.reload()
 
         """
-        c_vars, d_vars, s_deps = self._get_dependencies(vars, modules)
+        deps = self.get_dependencies(vars, modules)
         warnings.warn(
             "Selecting submodel, "
             "to run the full model again use model.reload()")
 
         # get set of all dependencies and all variables to select
-        all_deps = d_vars["initial"].copy()
-        all_deps.update(d_vars["step"])
-        all_deps.update(d_vars["lookup"])
+        all_deps = deps.d_deps["initial"].copy()
+        all_deps.update(deps.d_deps["step"])
+        all_deps.update(deps.d_deps["lookup"])
 
         all_vars = all_deps.copy()
-        all_vars.update(c_vars)
+        all_vars.update(deps.c_vars)
 
         # clean dependendies and namespace dictionaries, and remove
         # the rows from the documentation
@@ -1312,27 +1593,27 @@ class Model(Macro):
                 )
 
         for py_name in self._dependencies.copy().keys():
-            if py_name.startswith("_") and py_name not in s_deps:
+            if py_name.startswith("_") and py_name not in deps.s_deps:
                 del self._dependencies[py_name]
 
         # remove active initial from s_deps as they are "fake" objects
         # in dependencies
-        s_deps = {
-            dep for dep in s_deps if not dep.startswith("_active_initial")
+        deps.s_deps = {
+            dep for dep in deps.s_deps if not dep.startswith("_active_initial")
         }
 
         # reassing the dictionary and lists of needed stateful objects
         self._stateful_elements = {
             name: getattr(self.components, name)
-            for name in s_deps
+            for name in deps.s_deps
             if isinstance(getattr(self.components, name), Stateful)
         }
         self._dynamicstateful_elements = [
-            getattr(self.components, name) for name in s_deps
+            getattr(self.components, name) for name in deps.s_deps
             if isinstance(getattr(self.components, name), DynamicStateful)
         ]
         self._macro_elements = [
-            getattr(self.components, name) for name in s_deps
+            getattr(self.components, name) for name in deps.s_deps
             if isinstance(getattr(self.components, name), Macro)
         ]
 
@@ -1392,8 +1673,8 @@ class Model(Macro):
 
         Returns
         -------
-        dependencies: set
-            Set of dependencies nedded to run vars.
+        dependencies: pysd.py_backend.utils.Dependencies
+            Dependencies data object.
 
         Notes
         -----
@@ -1402,15 +1683,15 @@ class Model(Macro):
 
         Examples
         --------
-        >>> model.get_dependencies(
-        ...     vars=["Room Temperature", "Teacup temperature"])
+        >>> print(model.get_dependencies(
+        ...     vars=["Room Temperature", "Teacup temperature"]))
         Selected variables (total 1):
             room_temperature, teacup_temperature
         Stateful objects integrated with the selected variables (total 1):
             _integ_teacup_temperature
 
-        >>> model.get_dependencies(
-        ...     modules=["view_1", "view_2/subview_1"])
+        >>> print(model.get_dependencies(
+        ...     modules=["view_1", "view_2/subview_1"]))
         Selected variables (total 4):
             var1, var2, stock1, delay1
         Dependencies for initialization only (total 1):
@@ -1420,9 +1701,9 @@ class Model(Macro):
         Stateful objects integrated with the selected variables (total 1):
             _integ_stock1, _delay_fixed_delay1
 
-        >>> model.get_dependencies(
+        >>> print(model.get_dependencies(
         ...     vars=["stock3"],
-        ...     modules=["view_1", "view_2/subview_1"])
+        ...     modules=["view_1", "view_2/subview_1"]))
         Selected variables (total 4):
             var1, var2, stock1, stock3, delay1
         Dependencies for initialization only (total 1):
@@ -1431,95 +1712,46 @@ class Model(Macro):
             _integ_stock1, _integ_stock3, _delay_fixed_delay1
 
         """
-        c_vars, d_vars, s_deps = self._get_dependencies(vars, modules)
-
-        text = utils.print_objects_format(c_vars, "Selected variables")
-
-        if d_vars["initial"]:
-            text += utils.print_objects_format(
-                d_vars["initial"],
-                "\nDependencies for initialization only")
-        if d_vars["step"]:
-            text += utils.print_objects_format(
-                d_vars["step"],
-                "\nDependencies that may change over time")
-        if d_vars["lookup"]:
-            text += utils.print_objects_format(
-                d_vars["lookup"],
-                "\nLookup table dependencies")
-
-        text += utils.print_objects_format(
-            s_deps,
-            "\nStateful objects integrated with the selected variables")
-
-        print(text)
-
-    def _get_dependencies(self, vars=[], modules=[]):
-        """
-        Get the dependencies of a set of variables or modules.
-
-        Parameters
-        ----------
-        vars: set or list of strings (optional)
-            Variables to get the dependencies from.
-            It can be an empty list if the dependencies are computed only
-            using modules. Default is an empty list.
-        modules: set or list of strings (optional)
-            Modules to get the dependencies from.
-            It can be an empty list if the dependencies are computed only
-            using variables. Default is an empty list. Can select a full
-            module or a submodule by passing the path without the .py, e.g.:
-            "view_1/submodule1".
-
-        Returns
-        -------
-        c_vars: set
-            Set of all selected model variables.
-        d_deps: dict of sets
-            Dictionary of dependencies nedded to run vars and modules.
-        s_deps: set
-            Set of stateful objects to update when integrating selected
-            model variables.
-
-        """
-        def check_dep(dependencies, initial=False):
-            for dep in dependencies:
-                if dep in c_vars or dep.startswith("__"):
+        def check_dep(deps_obj, deps, initial=False):
+            for dep in deps:
+                if dep in deps_obj.c_vars or dep.startswith("__"):
                     pass
                 elif dep.startswith("_"):
-                    s_deps.add(dep)
+                    deps_obj.s_deps.add(dep)
                     dep = self._dependencies[dep]
-                    check_dep(dep["initial"], True)
-                    check_dep(dep["step"])
+                    check_dep(deps_obj, dep["initial"], True)
+                    check_dep(deps_obj, dep["step"])
                 else:
-                    if initial and dep not in d_deps["step"]\
-                       and dep not in d_deps["lookup"]:
-                        d_deps["initial"].add(dep)
+                    if initial and dep not in deps_obj.d_deps["step"]\
+                       and dep not in deps_obj.d_deps["lookup"]:
+                        deps_obj.d_deps["initial"].add(dep)
                     else:
-                        if dep in d_deps["initial"]:
-                            d_deps["initial"].remove(dep)
+                        if dep in deps_obj.d_deps["initial"]:
+                            deps_obj.d_deps["initial"].remove(dep)
                         if self.get_args(dep):
-                            d_deps["lookup"].add(dep)
+                            deps_obj.d_deps["lookup"].add(dep)
                         else:
-                            d_deps["step"].add(dep)
+                            deps_obj.d_deps["step"].add(dep)
 
-        d_deps = {"initial": set(), "step": set(), "lookup": set()}
-        s_deps = set()
-        c_vars = {"time", "time_step", "initial_time", "final_time", "saveper"}
+        dependencies = utils.Dependencies(
+            {"time", "time_step", "initial_time", "final_time", "saveper"},
+            {"initial": set(), "step": set(), "lookup": set()},
+            set()
+        )
         for var in vars:
             py_name = utils.get_key_and_value_by_insensitive_key_or_value(
                     var,
                     self._namespace)[1]
-            c_vars.add(py_name)
+            dependencies.c_vars.add(py_name)
         for module in modules:
-            c_vars.update(self.get_vars_in_module(module))
+            dependencies.c_vars.update(self.get_vars_in_module(module))
 
-        for var in c_vars:
+        for var in dependencies.c_vars:
             if var == "time":
                 continue
-            check_dep(self._dependencies[var])
+            check_dep(dependencies, self._dependencies[var])
 
-        return c_vars, d_deps, s_deps
+        return dependencies
 
     def get_vars_in_module(self, module):
         """
